%Arbitrary Sample Rate Conversion (ASRC) by Lagrange interpolation
%using modified Candan's method
%
%Usage:
%[y t N t0 x0] = asrc_lagrange(x, dt, t0, x0, K)
%
%Where:
%   x  = vector of input samples at times [1:length(x)]
%   y  = vector of output samples at times t
%   t  = vector of zero delay output sample times relative to the input sample
%        times (i.e. as though the filter is non-causal)
%   N  = number of samples returned in y and t
%   dt = time step between output samples
%   t0 = time of first output sample in interval [0,dt) on input
%      = time of next first output sample
%   x0 = previous K samples of on input, i.e. [x(-K+1) ... x(0)]
%      = next x0 on output
%   K  = Lagrange order (K+1 tap filter), should be odd for a continuous filter
%
%The algorithm used is essentially that proposed by C. Candan, "An Efficient
%Filtering Structure for Lagrange Interpolation", IEEE Signal Processing
%Letters, Vol. 14, No. 1, January 2007 but with the delay coefficients moved
%from the input path to the output path similar to a conventional Farrow
%structure which seems to have several advantages over that proposed by Candan.
%
%Matt Boytim - 9/9/08
%maboytim@yahoo.com

function [y,t,N,t0,x0] = asrc_lagrange(x,dt,t0,x0,K)

% the time interval of [x0 x] is assumed to be (1-K):Nx
Nx = length(x);

% number of output points in the in the interval [0:Nx)
N = ceil((Nx-t0)/dt);

% and their sample times
t = t0+[0:N-1]*dt;

% compute input sample times from output sample times
% floor(.)+1 is essentially ceil(.) but with a different boundary case
T = floor(t)+1;

% x(T) is the newest sample involved in the interpolation
% the delay is relatve to the newest sample
% delay is in one sample span in the middle of the filter
% T-K      T-4      T-3      T-2      T-1       T
%  +--------+--------+---x----+--------+---o----+
%                                          t
%                                          |<-->|
%                                            T-t
%                        |<--------------->|
%                              (K-1)/2
%                        |<-------------------->|
%                           d = (K-1)/2 + (T-t)
t = t - (K-1)/2;
%t = t - floor((K-1)/2);
d = T-t;

% compute the delay coefficients
D = [];
for i=1:K,
  D = [D; (-d+i-1)/i];
end;

% compute K derivatives
dx = [x0 x];
Dx = [];
for i=1:K,
  dx = dx(2:Nx+K+1-i)-dx(1:Nx+K-i);
  Dx = [Dx; dx(K+1-i:Nx+K-i)];
end;

% compute the output iteration
y = zeros(1,N);
for i=K:-1:1,
  y = D(i,:).*(y + Dx(i,T));
end;
y = y + x(T);

% compute t0,x0 for next time
t0 = t0+N*dt - Nx;
x0 = x(Nx-K+1:Nx);
